function D2 = laplace_dis(nx, dx, type)
%LAPLACE_DIS discrete 1D Laplace operator, i.e. %d./dx.
%   Returns D2, a square matrix of size (nx,nx) 
%   such that for a vector y, D2*y approximates dy/dx.
%
%   Example: for nx=4, D2 = (1/dx^2) *
%   [-2     1     0     0
%     1    -2     1     0
%     0     1    -2     1
%     0     0     1    -2]
%
%   D2 is dense if type='dense'
%   D2 is sparse if type='sparse'
    if type=="dense"
        D2 = zeros(nx);
        for i=1:nx
            D2(i,i) = -2/dx^2;
            if i>=2
                D2(i,i-1) = 1/dx^2;
            end
            if i<=nx-1
                D2(i,i+1) = 1/dx^2;
            end
        end % for i
    elseif type=="sparse"
        D2diags = [ones(nx,1) -2*ones(nx,1) ones(nx,1)]/dx^2;
        D2 = spdiags(D2diags, [-1, 0, 1], nx, nx);
    else
        ME = MException('Laplace:BadMatrixType', 'type argument should be "dense" or "sparse"');
        throw(ME);
    end
end