% Comparison of execution time of classical operations of linear algebra
% with dense vs. sparse matrices:
% * Matrix-vector product A.x
% * Solve A.x = b
% * eigs

string_params;

%% Matrix vector product

nx = 10000;
dx = 1;

D2 = laplace_dis(nx, dx, "dense");
D2s = laplace_dis(nx, dx, "sparse");

x = ones(nx,1);
f1 = @() D2*x;
f2 = @() D2s*x;

% timing of dense vs. sparse product
time_fun(f1, 1)
time_fun(f2, 1)

%% Inversion of linear system (from Bode diagram)

w = 2*pi*100;
M = (-w^2 + 1j*w*a/mu)*eye(nx) - T/mu*D2;
Ms = (-w^2 + 1j*w*a/mu)*speye(nx) - T/mu*D2s;
u = ones(nx,1);

f1 = @() abs(M \ u);
f2 = @() abs(Ms \ u); 

% timing of dense vs. sparse linear system inversion
time_fun(f1, 1)
time_fun(f2, 1)

%% Eigen values (from mode analysis)
% NB: with anonymous functions, it seems unfortunately impossible
% to time the "[V,D]=eig..." syntax where gives eigenvectors...

n_modes = 10;

% Classical eig, using dense matrix:
f1 = @() eig(T/mu*D2);
% Compute only a few eigenvalues, using sparse matrix
f2 = @() eigs(T/mu*D2s, n_modes, 'smallestabs');
% Compute only a few eigenvalues, using dense matrix
f3 = @() eigs(T/mu*D2, n_modes, 'smallestabs'); 

time_fun(f1, 1)
time_fun(f2, 1)
time_fun(f3, 1)